/*
 *******************************************************************
 *              TMark favorites plugin for TVBrowser               *
 *                                                                 *
 * Copyright (C) 2006 Tomas Schackert.                             *
 * Contact koumori@web.de                                          *
 *******************************************************************

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, in version 2 of the License.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program, in a file called LICENSE in the top
 directory of the distribution; if not, write to 
 the Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA  02111-1307  USA
 
 *******************************************************************/

package tmarkplugin.properties;

import java.awt.Color;
import java.awt.Font;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.text.Style;
import javax.swing.text.StyleConstants;


/**
 * A Property is normally a set of simple properties that are combined to store more complex data.
 * A Property allows to be shared between a group of objects which get notified if the value changes.
 * An example for objects sharing a property are the interaction of a GUI-component and its setting dialog,
 * changing the settings will cause the component to update its visualisation. 
 * subclasses also should support a valid way to copy values from another instance to allow the mainprogram
 * a simple handling with more then one value set.
 * @author Tomas
 *
 */
abstract public class Property{
	private PropertySource src;
	
	public interface Listener{
		public void changedProperty(Property p);
	}
	Vector listener=new Vector();
	String name="";
	
	public Property(PropertySource src){
		this.src=src;
	}
	
	public PropertySource getSource(){
		return src;
	}
	
	public void addListener(Listener l){
		listener.add(l);
	}
	public void removeListener(Listener l){
		listener.remove(l);
	}
	private boolean fireingChanged=false;
	synchronized protected void fireChanged(){
		if (fireingChanged) return;
		fireingChanged=true;
		Iterator it=new Vector(listener).iterator();
		while (it.hasNext()){
			Listener l=(Listener)it.next();
			l.changedProperty(this);
		}
		fireingChanged=false;
	}
	public String getName(){
		return name;
	}
	public void setName(String name){
		this.name=name;
	}
	abstract public void reset();
	
	/** dynamic access to data types*/
	public int getIntProperty(String key,int defaultValue){
		if (src!=null)return src.getIntProperty(key,defaultValue);
		else return defaultValue;
	}
	public void setIntProperty(String key,int value) {
		if (src!=null)src.setIntProperty(key,value);
	}
	public void setIntArrayProperty(String key,int value[]){
		if (src!=null)src.setIntArrayProperty(key,value);
	}
	public int[] getIntArrayProperty(String key,int defaultValue[]){
		if (src!=null)return src.getIntArrayProperty(key,defaultValue);
		else return defaultValue;
	}
	public void setStringArrayProperty(String key,String value[]){
		if (src!=null)src.setStringArrayProperty(key,value);
	}
	public String[] getStringArrayProperty(String key,String defaultValue[]){
		if (src!=null)return src.getStringArrayProperty(key,defaultValue);
		else return defaultValue;
	}
	
	public void setBoolProperty(String key,boolean value) {
		if (src!=null)src.setBoolProperty(key,value);
	}
	public boolean getBoolProperty(String key,boolean defaultValue){
		if (src!=null)return src.getBoolProperty(key,defaultValue);
		else return defaultValue;
	}

	public void setFloatProperty(String key,float value) {
		if (src!=null)src.setFloatProperty(key,value); 
	}
	public float getFloatProperty(String key,float defaultValue){
		if (src!=null)return src.getFloatProperty(key,defaultValue);
		else return defaultValue;
	}

	public void setLongProperty(String key,long value) {
		if (src!=null)src.setLongProperty(key,value);
	}

	public long getLongProperty(String key,long defaultValue){
		if (src!=null)return src.getLongProperty(key,defaultValue);
		else return defaultValue;
	}
	public String setProperty(String key, String value){
		if (src!=null)return src.setProperty(key,value);
		return null;
	}
	public String getProperty(String key, String defaultValue){
		if (src!=null)return src.getProperty(key,defaultValue);
		else return defaultValue;
	}
	public Enumeration getPropertyKeys(){
		if (src!=null)return src.getPropertyKeys();
		else return new Enumeration(){
			public boolean hasMoreElements() {
				return false;
			}
			public Object nextElement() {
				return null;
			}
		};
	}
	public Color getColorProperty(String key, Color defaultValue) {
		if (src!=null)return src.getColorProperty(key,defaultValue);
		else return defaultValue;
	}

	public void setColorProperty(String key, Color color){
		if (src!=null)src.setColorProperty(key,color);
	}
	public Font getFontProperty(String key, Font defaultValue){
		if (src!=null)return src.getFontProperty(key,defaultValue);
		else return defaultValue;
	}
	public void setFontProperty(String key, Font value){
		if (src!=null)src.setFontProperty(key,value);
	}



	/*utilities*/	
	static public Font getFontFromStyle(Style style){
		int s=0;
		if (StyleConstants.isBold(style))s|=Font.BOLD;
		if (StyleConstants.isItalic(style))s|=Font.ITALIC;
		String name=StyleConstants.getFontFamily(style);
		int size=StyleConstants.getFontSize(style);
		return new Font(name, s, size);
	}
	static public void setFontForStyle(Style style,Font font){
		StyleConstants.setFontFamily(style,font.getFamily());
		StyleConstants.setBold(style,font.isBold());
		StyleConstants.setItalic(style,font.isItalic());
	}
}
